from collections import OrderedDict

from jet_django.deps.rest_framework.pagination import PageNumberPagination
from jet_django.deps.rest_framework.response import Response


class CustomPageNumberPagination(PageNumberPagination):
    page_size_query_param = '_per_page'

    def get_paginated_response(self, data):
        return Response(OrderedDict([
            ('count', self.page.paginator.count),
            ('next', self.get_next_link()),
            ('previous', self.get_previous_link()),
            ('results', data),
            ('num_pages', self.page.paginator.num_pages),
            ('per_page', self.page.paginator.per_page),
        ]))
